<?php

namespace Drupal\document_ocr\Entity\Form\Mapping;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Process Documents in bulk form.
 */
class ProcessForm extends ConfirmFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $currentRouteMatch;

  /**
   * Constructs a new BulkImportForm.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RouteMatchInterface $current_route_match) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentRouteMatch = $current_route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'document_ocr_bulk_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $document_ocr_mapping = $this->currentRouteMatch->getParameter('document_ocr_mapping');
    return $this->t('Process pending files for %name?', ['%name' => $document_ocr_mapping->label()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Process Files (@count)', ['@count' => $this->getPendingItemsCount()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return Url::fromRoute('entity.document_ocr_mapping.collection');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $form['#attached']['library'][] = 'document_ocr/document_ocr.admin';
    $form_state->set('document_ocr_mapping', $this->currentRouteMatch->getParameter('document_ocr_mapping'));
    $form['actions']['cancel']['#attributes']['class'][] = 'dialog-cancel';
    $form['description']['#markup'] = $this->t('This action lookup unprocessed files (<em>including entities created prior Document OCR module installation</em>) and import into destination entities. Depending on file sizes this may take time please keep the window open. <br/><span class="form-item__description">Failed documents will be added to the re-try queue and processed automatically on cron.</span>');
    $form['description']['#prefix'] = '<p>';
    $form['description']['#suffx'] = '</p>';
    if (!$this->getPendingItemsCount()) {
      $form['actions']['empty']['#markup'] = $this->t('Looks like everything is already procssed and there are no pending files.');
    }
    $form['actions']['submit']['#prefix'] = '<div class="document-ocr-button-wrapper"><span class="document-ocr-button-spinner"></span>';
    $form['actions']['submit']['#suffix'] = '</div>';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $document = $form_state->get('document_ocr_mapping');
    $source_entity = $document->getFieldSelector();
    $batch = [
      'title' => $this->t('Processing pending files'),
      'finished' => '\Drupal\document_ocr\Services\Batch::completeProcessCallback',
      'operations' => [],
    ];
    $storage = $this->entityTypeManager->getStorage($source_entity['entity_type_id']);
    $query = $storage->getQuery();
    $unprocessed_ids = $query->accessCheck(FALSE)->execute();
    $entities = $storage->loadMultiple($unprocessed_ids);
    foreach ($entities as $entity) {
      if ($source_entity['bundle'] == $entity->bundle()) {
        $item = new \stdClass();
        $item->document = $document;
        $item->entity = $entity;
        $batch['operations'][] = [
          '\Drupal\document_ocr\Services\Batch::processOlderEntity', [$item],
        ];
      }
    }
    $storage = $this->entityTypeManager->getStorage('document_ocr_import');
    $query = $storage->getQuery()
      ->condition('document_ocr_mapping', $document->id())
      ->condition('processed', 0);
    $ids = $query->accessCheck(FALSE)->execute();
    $entities = $storage->loadMultiple($ids);
    foreach ($entities as $import) {
      $item = new \stdClass();
      $item->id = $import->id();
      $batch['operations'][] = [
        '\Drupal\document_ocr\Services\Batch::process', [$item],
      ];
    }
    batch_set($batch);
    $form_state->setRedirect('entity.document_ocr_mapping.collection');
  }

  /**
   * Get the number of pending document items.
   */
  protected function getPendingItemsCount() {
    $count = 0;
    $document_ocr_mapping = $this->currentRouteMatch->getParameter('document_ocr_mapping');
    $settings = $document_ocr_mapping->getSettings();
    $query = $this->entityTypeManager->getStorage('document_ocr_import')->getQuery()
      ->condition('document_ocr_mapping', $document_ocr_mapping->id())
      ->condition('processed', 0)
      ->condition('attempts', $settings['attempts'], '<')
      ->accessCheck(FALSE);
    $count += (int) $query->count()->execute();

    $source_entity = $document_ocr_mapping->getFieldSelector();
    $query2 = $this->entityTypeManager->getStorage($source_entity['entity_type_id'])->getQuery()
      ->condition('type', $source_entity['bundle'])
      ->condition('status', 1)
      ->accessCheck(FALSE);
    $count += (int) $query2->count()->execute();
    return $count;
  }

}
